/*
 * KMLOFax
 *
 * A utility to process facsimiles received with a modem of the
 * ELSA MicroLink Office family.
 *
 * Copyright (C) 1999,2000,2001,2002 Oliver Gantz <Oliver.Gantz@epost.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef GLOBAL_H
#define GLOBAL_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif 

#include <qstring.h>
#include <qdatetime.h>


#define PAPER_A4        0
#define PAPER_B4        1
#define PAPER_B5        2
#define PAPER_LETTER    3
#define PAPER_LEGAL     4
#define PAPER_EXECUTIVE 5

#define PAPER_PORTRAIT  0
#define PAPER_LANDSCAPE 1

#define INBOX_DIR								"inbox"
#define TRASH_DIR								"trash"
#define FOLDERS_DIR							"folders"
#define TEMP_DIR								"tmp"

#define KMLOFAX_INBOX						"Inbox"
#define KMLOFAX_TRASH						"Trash"
#define KMLOFAX_FAXDIR					"facsimilerc"
#define KMLOFAX_ALIASLIST				"aliasesrc"

#define ID_PREF_GROUP_GENERAL		"General"
#define ID_PREF_GEN_TOOLTIPS		"ToolTips"
#define PREF_GEN_TOOLTIPS				1
#define ID_PREF_GROUP_PREVIEW 	"Preview"
#define ID_PREF_GEN_VIEWMODE		"ViewMode"
#define PREF_GEN_VIEWMODE				1
#define ID_PREF_GEN_VIEWER			"ExternalViewer"
#define PREF_GEN_VIEWER					"kfax"

#define ID_PREF_GROUP_MODEM			"Modem"
#define ID_PREF_MOD_USB					"UseUSB"
#define PREF_MOD_USB		   			0
#define ID_PREF_MOD_SER_DEVICE	"SerialDevice"
#define PREF_MOD_SER_DEVICE  		0		// == /dev/modem
#define ID_PREF_MOD_SER_SPEED		"SerialSpeed"
#define PREF_MOD_SER_SPEED   		1		// == 38400 Bps
#define ID_PREF_MOD_USB_DEVICE	"USBDevice"
#define PREF_MOD_USB_DEVICE  		0		// == /dev/usb/ttyACM0
#define ID_PREF_MOD_LOADNEW			"DownloadNew"
#define PREF_MOD_LOADNEW				0
#define ID_PREF_MOD_DELETEFAX		"DeleteFax"
#define PREF_MOD_DELETEFAX			1

#define ID_PREF_GROUP_PRINT			"Print"
#define ID_PREF_PRT_PRINTER			"Printer"
#define ID_PREF_PRT_HOST				"Host"
#define ID_PREF_PRT_LEVEL2			"LanguageLevel2"
#define PREF_PRT_LEVEL2					1
#define ID_PREF_PRT_INTERPOL		"Interpolate"
#define PREF_PRT_INTERPOL				0
#define ID_PREF_PRT_ORIENT			"Orientation"
#define PREF_PRT_ORIENT					PAPER_PORTRAIT
#define ID_PREF_PRT_PAPER				"Paper"
#define PREF_PRT_PAPER					PAPER_A4
#define ID_PREF_PRT_COPIES			"Copies"
#define PREF_PRT_COPIES					1
#define ID_PREF_PRT_RIBBONS			"Ribbons"
#define PREF_PRT_RIBBONS				1

#define ID_PREF_PRT_LMARGIN			"LeftMargin"
#define PREF_PRT_LMARGIN				0
#define ID_PREF_PRT_RMARGIN			"RightMargin"
#define PREF_PRT_RMARGIN  			0
#define ID_PREF_PRT_TMARGIN			"TopMargin"
#define PREF_PRT_TMARGIN  			0
#define ID_PREF_PRT_BMARGIN			"BottomMargin"
#define PREF_PRT_BMARGIN  			0

#define ID_PREF_GROUP_MAIL			"Mail"
#define ID_PREF_MAIL_SERVER			"Server"
#define PREF_MAIL_SERVER				"localhost"
#define ID_PREF_MAIL_PORT				"Port"
#define PREF_MAIL_PORT					25
#define ID_PREF_MAIL_NAME				"Name"
#define PREF_MAIL_NAME					""
#define ID_PREF_MAIL_ORG				"Organization"
#define PREF_MAIL_ORG						""
#define ID_PREF_MAIL_EMAIL			"Email"
#define PREF_MAIL_EMAIL					""
#define ID_PREF_MAIL_REPLY			"Reply"
#define PREF_MAIL_REPLY					""

#define ID_PREF_GROUP_PREVIEW			"Preview"
#define ID_PREF_PREVIEW_INTERNAL	"Internal"
#define PREF_PREVIEW_INTERNAL			1


#define ID_FILE_DOWNLOAD           10010
#define ID_FILE_VIEW               10020
#define ID_FILE_PRINT              10030
#define ID_FILE_EXPORT             10040
#define ID_FILE_MAIL               10050
#define ID_FILE_TRASH              10060
#define ID_FILE_SENDER_ALIAS       10070
#define ID_FILE_REBUILD_DIRECTORY  10080
#define ID_FILE_EMPTY_TRASH        10090
#define ID_FILE_QUIT               10100

#define ID_FOLDER_NEW              11010
#define ID_FOLDER_RENAME           11020
#define ID_FOLDER_EMPTY            11030
#define ID_FOLDER_REMOVE           11040

#define ID_SETTINGS_TOOLBAR        12010
#define ID_SETTINGS_STATUSBAR      12020
#define ID_SETTINGS_SAVEOPTIONS    12030
#define ID_SETTINGS_SENDER_ALIASES 12040
#define ID_SETTINGS_CONFIGURE      12050

#define ID_HELP_CONTENTS           1002
#define ID_HELP_ABOUT              1003

#define ID_STATUS_MSG              1001



extern const uchar xchg_endian[];

QString expandPath(const QString &fname);

QString mloDateStr(const QString &name);

QDateTime mloDateSplit(const QString &name);


#endif // GLOBAL_H
